#ifndef FBS_EDGE_DATABASE_H
#define FBS_EDGE_DATABASE_H

#include <stdint.h>
#include <vector>
#include <cassert>
#include <tcutil.h>
#include <tchdb.h>

#include <iostream>
#include "maxlink.h"
#include "maxlink_types.h"
#include "tcb_tokyo_cabinet_database.h"

int tccmpFbs(const char *ptr1, int size1, const char *ptr2, int size2, void *op);

class FbsDatabase : public TcbTokyoCabinetDatabase {
  public:
  FbsDatabase() {};
  ~FbsDatabase() {};
  void storeFbs(maxlink_types::Fbs fbs) {
    xml_schema::long_ position = 0; /* 0 is a dummy value. The position will later be calculated */
    assert(init_run_);
    assert(tcb_);
    if (!tcbdbputdup(tcb_, &fbs, sizeof(fbs), &position, sizeof(position))) {
      int error_code = tcbdbecode(tcb_);
      fprintf(stderr, "error in tcdbdput: %s\n", tcbdberrmsg(error_code));
    }
  }
  virtual void init() {
    TcbTokyoCabinetDatabase::init();
    bool ret = tcbdbsetcmpfunc(tcb_, tccmpFbs, NULL);
    assert(ret);
  }
  std::string endingFilename() {
    return std::string("fbs.tcb");
  };
  uint64_t num_edges_over_cutoff(maxlink_types::Fbs fbs) {
    uint64_t total = tcbdbrnum(tcb_);
    BDBCUR *cur;
    cur = tcbdbcurnew(tcb_);
    bool ret = tcbdbcurjump(cur, &fbs, sizeof(fbs));
    uint64_t cutoff_count = total;
    if (ret) {
      int size;
      size_t expected_size = sizeof(xml_schema::long_ *);
      xml_schema::long_ *result_ptr = static_cast<xml_schema::long_ *>(tcbdbcurval(cur, &size));
      assert(size == expected_size);
      assert(result_ptr);
      cutoff_count = *result_ptr;
      free(result_ptr);
    }
    tcbdbcurdel(cur);   
    return cutoff_count;
  }
  void calculatePositions() {
    // As the compare function tccmpFbs() sorts in reversed order, the position stored
    // will be the same as the number of keys with a higher cutoff.
    BDBCUR *cur;
    cur = tcbdbcurnew(tcb_);
    tcbdbcurfirst(cur);
    xml_schema::long_  position = 0;
    do {
      if (!tcbdbcurput(cur, &position, sizeof(position), BDBCPCURRENT)) {
      int error_code = tcbdbecode(tcb_);
      fprintf(stderr, "error in tcbdbput: %s\n", tcbdberrmsg(error_code));
      }
      ++position;
    } while (tcbdbcurnext(cur));
    tcbdbcurdel(cur);
  }
};

#endif
