#ifndef MAXLINK_TYPES_H_
#define MAXLINK_TYPES_H_

#include <vector>
#include "maxlink.h"

namespace maxlink_types {
// Chapter 5
// Mapping for Built-In XML Schema Types
// http://codesynthesis.com/projects/xsde/documentation/cxx/hybrid/guide/#5
//
// XML Schema type: unsignedLong  
// is mapped to C++ type:
// xml_schema::unsigned_long

typedef int GeneId;
typedef xml_schema::float_ Fbs;
typedef xml_schema::float_ Pfc;
typedef xml_schema::unsigned_long ShortestPathId;
typedef std::vector<GeneId> ShortestPath;

typedef struct { 
  GeneId gene_id; 
  Fbs fbs;
  Pfc pfc;} HalfEdge;

typedef struct { 
  GeneId first; 
  GeneId second; 
} GenePair;

typedef std::vector<GeneId>::size_type GeneCount;

struct GeneIdScore {
  GeneId gene_id;
  GeneCount maxlink_score;
  GeneCount non_q_neighbours_count;
  double hypergeometric_probability;
};

struct GeneIdHyperGeometricResult {
  maxlink_types::GeneId gene_id;
  double hypergeometric_probability;
  // The next variables are named after the nomenclatur in:
  // http://www.gnu.org/software/gsl/manual/html_node/The-Hypergeometric-Distribution.html
  // "If a population contains n_1 elements type  and n_2 elements type  then the 
  // hypergeometric distribution gives the probability of obtaining k elements type 
  // in t samples from the population without replacement.1of 2of 1of"
  unsigned int k;
  unsigned int t;
};
} 

#endif

