#ifndef TCB_TOKYO_CABINET_DATABASE_H
#define TCB_TOKYO_CABINET_DATABASE_H

#include <cassert>
#include <tcutil.h>
#include <tcbdb.h>
#include <cstdlib>
#include <stdint.h>

class TcbTokyoCabinetDatabase {
 public:
TcbTokyoCabinetDatabase() : tcb_(NULL), init_run_(false) {};
  ~TcbTokyoCabinetDatabase() {
    if(!tcbdbclose(tcb_)){
      int errorcode = tcbdbecode(tcb_);
     fprintf(stderr, "close error: %s\n", tcbdberrmsg(errorcode));
    }
    tcbdbdel(tcb_);
  };
  virtual void init() {
    tcb_ = tcbdbnew();
    assert(tcb_);
    // For more informationen see http://www.1978th.net/tokyocabinet/spex-en.html#tcbdbapi
    bool ret = tcbdbtune(tcb_, 128 /*default*/, 256 /*default*/, 32749 /*default*/, 8 /*default*/, 10 /*default*/, BDBTLARGE);
    if (!ret) {
      int error_code = tcbdbecode(tcb_);
      fprintf(stderr, "error: tcbdbtune did not return 0: %s\n", tcbdberrmsg(error_code));
      exit(EXIT_FAILURE);
    }
    init_run_=true;
  }
  void openReadOnly(std::string tokyo_cabinet_dir) {
    init();
    if(!tcbdbopen(tcb_, fullFileName(tokyo_cabinet_dir).c_str(), BDBOREADER)){
      int error_code = tcbdbecode(tcb_);
      fprintf(stderr, "open error: %s\n", tcbdberrmsg(error_code));
      exit(EXIT_FAILURE);
    }
  }

  uint64_t num_entries() const {
    return tcbdbrnum(tcb_);
  }

  void create(std::string tokyo_cabinet_dir) {
    init();
    if(!tcbdbopen(tcb_, fullFileName(tokyo_cabinet_dir).c_str(), BDBOWRITER | BDBOCREAT)){
      int errorcode = tcbdbecode(tcb_);
      fprintf(stderr, "open error: %s\n", tcbdberrmsg(errorcode));
      exit(EXIT_FAILURE);
    }
  }
  std::string fullFileName(std::string tokyo_cabinet_dir) {
    std::string file_name = tokyo_cabinet_dir + std::string("/") + endingFilename();
    return file_name;
  }
  virtual std::string endingFilename() = 0;


 protected:
  TCBDB *tcb_;
  bool init_run_;
};

#endif
